@@ -10,6 +10,7 @@ class ScaffoldContext(dict):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self['steps'] = {}
         self.notes = []
         self.todos = []
         self.environ = {}
@@ -21,3 +22,6 @@ class ScaffoldContext(dict):
     def resolve_target_path(self, path):
         target_dir = self['__target']
         return os.path.realpath(os.path.join(target_dir, path))
+    
+    def set_step(self, step_id, value):
+        self['steps'][step_id] = value
