@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
+import collections
 import logging
 import os
 import re
@@ -212,7 +213,7 @@ def process_parameters(parameters, context: ScaffoldContext, runtime: ScaffoldRu
         parameter_options = render_options(parameter, context)
         parameter_name = parameter_options['name']
         if parameter_name in context:
-            context[parameter_name] = context[parameter_name]
+            context[parameter_name] = convert(context[parameter_name], parameter.get('type', 'str'))
         else:
             context[parameter_name] = read_parameter(parameter_options, context, runtime)
 
@@ -234,17 +235,20 @@ def execute_scaffold(context: ScaffoldContext, options, runtime: ScaffoldRuntime
 
     name = options.get('name', 'xscaffold')
 
-    if os.path.exists(package):
-        runtime.log(
-            '[info] using local package \'%s\'...' % package + '\n')
-        pkg_dir = package
+    if '__package' in context:
+        package_path = context.resolve_package_path(package)
+    else:
+        package_path = package
+    if os.path.exists(package_path):
+        _log.debug('using local package \'%s\'', package_path)
+        pkg_dir = package_path
     else:
         pkg_dir = fetch_git(runtime, tempdir, package)
     
     scaffold_file = locate_scaffold_file(pkg_dir, name)
     pkg_dir = os.path.dirname(scaffold_file)
     sys.path.append(pkg_dir)
-    _log.info('scaffold file: %s', scaffold_file)
+    _log.debug('scaffold file: %s', scaffold_file)
 
     if scaffold_file is not None:
         with open(scaffold_file, 'r') as fhd:
@@ -277,22 +281,22 @@ def execute_scaffold(context: ScaffoldContext, options, runtime: ScaffoldRuntime
 
     return context
 
-def execute_steps(context, runtime, plugin_context, steps_context, steps):
+def execute_steps(context: ScaffoldContext, runtime, plugin_context, steps_context, steps):
     step: dict
     for step in steps:
         if 'if' in step:
             enabled = render_text(step['if'], context)
             if enabled == False:
                 continue
-        step_id = step.get('id', 'last')
         if 'group' in step:
             group_steps = step['group']
             execute_steps(context, runtime, plugin_context, steps_context, group_steps)
         elif 'foreach' in step:
             foreach_steps = step['foreach']
             items = render_text(foreach_steps.get('items', []), context)
+            step_id = step.get('id', 'foreach')
             for item in items:
-                steps_context[step_id] = item
+                context.set_step(step_id, item)
                 execute_steps(context, runtime, plugin_context, steps_context, foreach_steps.get('steps', []))
         else:
             plugin_step_name = None
@@ -302,12 +306,16 @@ def execute_steps(context, runtime, plugin_context, steps_context, steps):
                     break
             if plugin_step_name:
                 plugin_step = plugin_context.steps[plugin_step_name]
-                result = plugin_step.run(context, step[plugin_step_name], runtime)
+                step_options = step[plugin_step_name]
 
-                if 'id' in step:
-                    step_id = step['id']
-                    steps_context[step_id] = result
+                step_id = step.get('id', plugin_step_name)
 
+                if isinstance(step_options, collections.Mapping):
+                    step_options['__id'] = step_id
+                
+                _log.debug(f'[{step_id}] running')
+                result = plugin_step.run(context, step_options, runtime)
+                context.set_step(step_id, result)
 
 def fetch_git(runtime, tempdir, package):
     package_parts = package.split('@')
